/* SPDX-License-Identifier: (GPL-2.0 OR MIT) */
/*
 * Copyright (c) 2022 MediaTek Inc.
 */

#ifndef _DT_BINDINGS_POWER_MT8365_POWER_H
#define _DT_BINDINGS_POWER_MT8365_POWER_H

#define MT8365_POWER_DOMAIN_MM		0
#define MT8365_POWER_DOMAIN_CONN	1
#define MT8365_POWER_DOMAIN_MFG		2
#define MT8365_POWER_DOMAIN_AUDIO	3
#define MT8365_POWER_DOMAIN_CAM		4
#define MT8365_POWER_DOMAIN_DSP		5
#define MT8365_POWER_DOMAIN_VDEC	6
#define MT8365_POWER_DOMAIN_VENC	7
#define MT8365_POWER_DOMAIN_APU		8

#endif /* _DT_BINDINGS_POWER_MT8365_POWER_H */
