/* SPDX-License-Identifier: (GPL-2.0-only OR BSD-2-Clause) */
/*
 * Copyright (c) 2021 MediaTek Inc.
 * Copyright (c) 2024 Collabora Ltd.
 *                    AngeloGioacchino Del Regno <angelogioacchino.delregno@collabora.com>
 */

#ifndef __DT_BINDINGS_INTERCONNECT_MEDIATEK_MT8183_H
#define __DT_BINDINGS_INTERCONNECT_MEDIATEK_MT8183_H

#define SLAVE_DDR_EMI		0
#define MASTER_MCUSYS		1
#define MASTER_MFG		2
#define MASTER_MMSYS		3
#define MASTER_MM_VPU		4
#define MASTER_MM_DISP		5
#define MASTER_MM_VDEC		6
#define MASTER_MM_VENC		7
#define MASTER_MM_CAM		8
#define MASTER_MM_IMG		9
#define MASTER_MM_MDP		10

#endif
