/* SPDX-License-Identifier: GPL-2.0-only */
/*
 * Arch specific extensions to struct device
 */
#ifndef _ASM_GENERIC_DEVICE_H
#define _ASM_GENERIC_DEVICE_H

struct dev_archdata {
};

struct pdev_archdata {
};

#endif /* _ASM_GENERIC_DEVICE_H */
