/* SPDX-License-Identifier: GPL-2.0 */
#ifndef _DT_BINDINGS_INTERRUPT_CONTROLLER_MIPS_GIC_H
#define _DT_BINDINGS_INTERRUPT_CONTROLLER_MIPS_GIC_H

#include <dt-bindings/interrupt-controller/irq.h>

#define GIC_SHARED 0
#define GIC_LOCAL 1

#endif
