/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Berlin2 BG2/BG2CD clock tree IDs
 */

#define CLKID_SYS		0
#define CLKID_CPU		1
#define CLKID_DRMFIGO		2
#define CLKID_CFG		3
#define CLKID_GFX		4
#define CLKID_ZSP		5
#define CLKID_PERIF		6
#define CLKID_PCUBE		7
#define CLKID_VSCOPE		8
#define CLKID_NFC_ECC		9
#define CLKID_VPP		10
#define CLKID_APP		11
#define CLKID_AUDIO0		12
#define CLKID_AUDIO2		13
#define CLKID_AUDIO3		14
#define CLKID_AUDIO1		15
#define CLKID_GFX3D_CORE	16
#define CLKID_GFX3D_SYS		17
#define CLKID_ARC		18
#define CLKID_VIP		19
#define CLKID_SDIO0XIN		20
#define CLKID_SDIO1XIN		21
#define CLKID_GFX3D_EXTRA	22
#define CLKID_GC360		23
#define CLKID_SDIO_DLLMST	24
#define CLKID_GETH0		25
#define CLKID_GETH1		26
#define CLKID_SATA		27
#define CLKID_AHBAPB		28
#define CLKID_USB0		29
#define CLKID_USB1		30
#define CLKID_PBRIDGE		31
#define CLKID_SDIO0		32
#define CLKID_SDIO1		33
#define CLKID_NFC		34
#define CLKID_SMEMC		35
#define CLKID_AUDIOHD		36
#define CLKID_VIDEO0		37
#define CLKID_VIDEO1		38
#define CLKID_VIDEO2		39
#define CLKID_TWD		40
